import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React, { useCallback, useContext } from 'react';
import isNil from 'lodash/isNil';
import { createChainedFunction, shallowEqual, useClassNames } from '../utils';
import { SidenavContext } from './Sidenav';
import PropTypes from 'prop-types';
import Ripple from '../Ripple';
import SafeAnchor from '../SafeAnchor';
import NavContext from '../Nav/NavContext';
import DropdownContext from '../Dropdown/DropdownContext';
var defaultProps = {
  as: 'li',
  classPrefix: 'dropdown-item'
};
/**
 * Tree View Node
 * @see https://www.w3.org/TR/wai-aria-practices-1.2/#TreeView
 */

var SidenavDropdownItem = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var Component = props.as,
      activeProp = props.active,
      children = props.children,
      disabled = props.disabled,
      divider = props.divider,
      panel = props.panel,
      className = props.className,
      style = props.style,
      classPrefix = props.classPrefix,
      icon = props.icon,
      eventKey = props.eventKey,
      onClick = props.onClick,
      onSelect = props.onSelect,
      rest = _objectWithoutPropertiesLoose(props, ["as", "active", "children", "disabled", "divider", "panel", "className", "style", "classPrefix", "icon", "eventKey", "onClick", "onSelect"]);

  var _useContext = useContext(SidenavContext),
      activeKey = _useContext.activeKey,
      onSidenavSelect = _useContext.onSelect;

  var nav = useContext(NavContext);
  var dropdown = useContext(DropdownContext);

  var _useClassNames = useClassNames(classPrefix),
      merge = _useClassNames.merge,
      withClassPrefix = _useClassNames.withClassPrefix,
      prefix = _useClassNames.prefix;

  var selected = activeProp !== null && activeProp !== void 0 ? activeProp : !isNil(eventKey) && (shallowEqual(eventKey, activeKey) || shallowEqual(nav === null || nav === void 0 ? void 0 : nav.activeKey, eventKey));
  var classes = merge(className, withClassPrefix({
    'with-icon': icon,
    active: selected,
    disabled: disabled
  }));
  var handleClick = useCallback(function (event) {
    if (disabled) return;
    onSelect === null || onSelect === void 0 ? void 0 : onSelect(eventKey, event);
    dropdown === null || dropdown === void 0 ? void 0 : dropdown.onSelect(eventKey, event);
    onSidenavSelect === null || onSidenavSelect === void 0 ? void 0 : onSidenavSelect(eventKey, event);
  }, [disabled, onSelect, onSidenavSelect, eventKey, dropdown]);
  var menuitemEventHandlers = {
    onClick: createChainedFunction(handleClick, onClick)
  };

  if (divider) {
    return /*#__PURE__*/React.createElement(Component, _extends({
      ref: ref,
      role: "separator",
      style: style,
      className: merge(prefix('divider'), className)
    }, rest));
  }

  if (panel) {
    return /*#__PURE__*/React.createElement(Component, _extends({
      ref: ref,
      role: "none presentation",
      style: style,
      className: merge(prefix('panel'), className)
    }, rest), children);
  }

  return /*#__PURE__*/React.createElement(SafeAnchor, _extends({
    ref: ref
  }, rest, {
    style: style,
    className: classes,
    "aria-current": selected || undefined
  }, menuitemEventHandlers), icon && /*#__PURE__*/React.cloneElement(icon, {
    className: prefix('menu-icon')
  }), children, /*#__PURE__*/React.createElement(Ripple, null));
});
SidenavDropdownItem.displayName = 'Sidenav.Dropdown.Item';
SidenavDropdownItem.defaultProps = defaultProps;
SidenavDropdownItem.propTypes = {
  as: PropTypes.elementType,
  expanded: PropTypes.bool,
  active: PropTypes.bool,
  divider: PropTypes.bool,
  panel: PropTypes.bool,
  disabled: PropTypes.bool,
  submenu: PropTypes.element,
  onSelect: PropTypes.func,
  onClick: PropTypes.func,
  icon: PropTypes.node,
  eventKey: PropTypes.any,
  className: PropTypes.string,
  style: PropTypes.object,
  children: PropTypes.node,
  classPrefix: PropTypes.string,
  tabIndex: PropTypes.number,
  title: PropTypes.node,
  onMouseOver: PropTypes.func,
  onMouseOut: PropTypes.func
};
export default SidenavDropdownItem;